# -*- coding: utf-8 -*-

"""
Zettel 2
 Aufgabe 2: Wettbewergsmodell
 Zu lösen ist ein System von zwei GDGl

 (1)      x'=ax(b-x-cy); y'=dy(e-y-fx); 


 mit Hilfe  vom klassischen RK4 Verfahren. 
 (1) beschreibt die Konkurenz von zwei Populationen x(t) und y(t) 
 um die gleiche beschrä̈nkte Ressource.
"""
import matplotlib.pyplot as plt
import numpy as np


plt.close("all")

a=0.004
b=50.0
c=0.75
d=0.001
e=100.0
f=3.0

def wett1(x,y,t):
    global a,b,c
    return a*x*(b-x-c*y)
	
	
def wett2(x,y,t):
    global d,e,f
    return d*y*(e-y-f*x)
	
def rk4(wett1,wett2,x0,y0,t0,Tend,h):
    '''
    Runge-Kutta Method of Order 4.
    Approximation of the solution of the IVP for the system of two ODEs
    Input- wett1 and wett2  are the functions to be integrated;
        -t0 and Tend are the initial and end time points;
       - x0 and y0 are initial conditions;
       - M is the number of steps;
       - h is the time step 
    '''
    M=(Tend-t0)/h
    x=np.zeros(M)
    y=np.zeros(M)
    t=np.zeros(M)
    x[0]=x0
    y[0]=y0
    t[0]=t0
    for j in range (int(M-1)):
        k1=wett1(x[j],y[j],t[j])
        c1=wett2(x[j],y[j],t[j])
        k2=wett1(x[j]+h*k1/2,y[j]+h*c1/2,t[j]+h/2)
        c2=wett2(x[j]+h*k1/2,y[j]+h*c1/2,t[j]+h/2)
        k3=wett1(x[j]+h*k2/2,y[j]+h*c2/2,t[j]+h/2)
        c3=wett2(x[j]+h*k2/2,y[j]+h*c2/2,t[j]+h/2)
        k4=wett1(x[j]+h*k3,y[j]+h*c3,t[j]+h)
        c4=wett2(x[j]+h*k3,y[j]+h*c3,t[j]+h)
        x[j+1]=x[j]+h*(k1+2*k2+2*k3+k4)/6
        y[j+1]=y[j]+h*(c1+2*c2+2*c3+c4)/6
	t[j+1] = t[j]+h

# plot x(t) and y(t); subplot(m,n,p): (m x n) grid and p'th subplot 
    plt.subplot(121)
    plt.xlabel("x",fontsize=16)
    plt.ylabel("y",fontsize=16)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16)
    plt.xlim([-1,81])
    plt.ylim([-1,103])		
    plt.plot(x,y,linewidth=3)
# phase space plot
    plt.subplot(122)
    plt.plot(t,x,linewidth=2)
    plt.plot(t,y,linewidth=2)
    plt.xlabel("t",fontsize=16)
    plt.ylabel("x,y",fontsize=16)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=16) 	
    plt.xlim([0,Tend])
    plt.ylim([-1,105])       


###
x1 = np.linspace(-1.0,80.0,40)
y1 = np.linspace(-1.0,103.0,40)
x1, y1=np.meshgrid(x1, y1)
vx=a*x1*(b-x1-c*y1)
vy=d*y1*(e-y1-f*x1)
###

fig1 = plt.figure(1)   

#Plot fixpoints of the system: (0,0) and (20,40) are unstable, (0,100) and (50,0) are stable
plt.subplot(121)
plt.plot(50,0,'ro',markersize=10,markeredgecolor='r',markeredgewidth=2)
plt.plot(0,100,'ro',markersize=10,markeredgecolor='r',markeredgewidth=2)
plt.plot(20,40, marker='o', markersize=10, markeredgecolor='r',mfc='None',markeredgewidth=2)
plt.plot(0,0,marker='o',markersize=10, markeredgecolor='r',mfc='None',markeredgewidth=2)

#plt.streamplot(x1,y1,vx,vy,color='b',linewidth=2)

'''
Plot solutions, corresponding to different initial conditions
'''
rk4(wett1,wett2,70.,20.,0.,180.,0.01)
rk4(wett1,wett2,10.,10.,0.,180.,0.01)
rk4(wett1,wett2,50,70,0,180,0.01)
rk4(wett1,wett2,10,50,0,180,0.01)
rk4(wett1,wett2,15,30,0,180,0.01)
rk4(wett1,wett2,7,50,0,180,0.01)
rk4(wett1,wett2,12,40,0,180,0.01)
rk4(wett1,wett2,70,60,0,180,0.01)
rk4(wett1,wett2,50,80,0,180,0.01)
rk4(wett1,wett2,50,65,0,180,0.01)
rk4(wett1,wett2,30,90,0,180,0.01)


#plt.savefig("Wettbewerb.pdf")
plt.show()            


